@@ -1,4 +1,5 @@
-from ...ocr.gsm_ocr_config import set_dpi_awareness
+from ...ocr.gsm_ocr_config import set_dpi_awareness, get_scene_ocr_config_path, OCRConfig, get_scene_ocr_config
+from ...util.electron_config import *
 
 try:
     import win32gui
@@ -43,7 +44,6 @@ import queue
 from datetime import datetime
 from PIL import Image, ImageDraw, UnidentifiedImageError
 from loguru import logger
-from pynput import keyboard
 from desktop_notifier import DesktopNotifierSync
 import psutil
 
@@ -58,6 +58,7 @@ from .screen_coordinate_picker import get_screen_selection
 from GameSentenceMiner.util.configuration import get_temporary_directory, get_config
 
 config = None
+last_image = None
 
 
 class ClipboardThread(threading.Thread):
@@ -309,9 +310,10 @@ class RequestHandler(socketserver.BaseRequestHandler):
 class TextFiltering:
     accurate_filtering = False
 
-    def __init__(self, lang="ja"):
+    def __init__(self, lang='ja'):
         from pysbd import Segmenter
-        self.segmenter = Segmenter(language=lang, clean=True)
+        self.initial_lang = get_ocr_language() or lang
+        self.segmenter = Segmenter(language=get_ocr_language(), clean=True)
         self.kana_kanji_regex = re.compile(r'[\u3041-\u3096\u30A1-\u30FA\u4E00-\u9FFF]')
         self.chinese_common_regex = re.compile(r'[\u4E00-\u9FFF]')
         self.english_regex = re.compile(r'[a-zA-Z0-9.,!?;:"\'()\[\]{}]')
@@ -349,8 +351,13 @@ class TextFiltering:
             self.classify = langid.classify
 
     def __call__(self, text, last_result):
-        orig_text = self.segmenter.segment(text)
+        lang = get_ocr_language()
+        if self.initial_lang != lang:
+            from pysbd import Segmenter
+            self.segmenter = Segmenter(language=get_ocr_language(), clean=True)
+            self.initial_lang = get_ocr_language()
 
+        orig_text = self.segmenter.segment(text)
         orig_text_filtered = []
         for block in orig_text:
             if "BLANK_LINE" in block:
@@ -419,7 +426,7 @@ class TextFiltering:
 
 
 class ScreenshotThread(threading.Thread):
-    def __init__(self, screen_capture_area, screen_capture_window, screen_capture_exclusions, screen_capture_only_active_windows, screen_capture_areas, screen_capture_on_combo):
+    def __init__(self, screen_capture_area, screen_capture_window, screen_capture_exclusions, screen_capture_areas, screen_capture_on_combo):
         super().__init__(daemon=True)
         self.macos_window_tracker_instance = None
         self.windows_window_tracker_instance = None
@@ -490,7 +497,6 @@ class ScreenshotThread(threading.Thread):
 
 
         if self.screencapture_mode == 2 or self.screen_capture_window:
-            self.screen_capture_only_active_windows = screen_capture_only_active_windows
             area_invalid_error = '"screen_capture_area" must be empty, "screen_N" where N is a screen number starting from 1, a valid set of coordinates, or a valid window name'
             if sys.platform == 'darwin':
                 if config.get_general('screen_capture_old_macos_api') or int(platform.mac_ver()[0].split('.')[0]) < 14:
@@ -523,7 +529,7 @@ class ScreenshotThread(threading.Thread):
                 self.window_id = window_ids[window_index]
                 window_title = window_titles[window_index]
 
-                if self.screen_capture_only_active_windows:
+                if get_ocr_requires_open_window():
                     self.macos_window_tracker_instance = threading.Thread(target=self.macos_window_tracker)
                     self.macos_window_tracker_instance.start()
                 logger.opt(ansi=True).info(f'Selected window: {window_title}')
@@ -567,7 +573,7 @@ class ScreenshotThread(threading.Thread):
             found = win32gui.IsWindow(self.window_handle)
             if not found:
                 break
-            if self.screen_capture_only_active_windows:
+            if get_ocr_requires_open_window():
                 self.screencapture_window_active = self.window_handle == win32gui.GetForegroundWindow()
             else:
                 self.screencapture_window_visible = not win32gui.IsIconic(self.window_handle)
@@ -656,7 +662,14 @@ class ScreenshotThread(threading.Thread):
     def run(self):
         if self.screencapture_mode != 2:
             sct = mss.mss()
+        start = time.time()
         while not terminated:
+            if time.time() - start > 1:
+                start = time.time()
+                section_changed = has_ocr_config_changed()
+                if section_changed:
+                    reload_electron_config()
+
             if not screenshot_event.wait(timeout=0.1):
                 continue
             if self.screencapture_mode == 2 or self.screen_capture_window:
@@ -719,6 +732,11 @@ class ScreenshotThread(threading.Thread):
             else:
                 sct_img = sct.grab(self.sct_params)
                 img = Image.frombytes('RGB', sct_img.size, sct_img.bgra, 'raw', 'BGRX')
+                
+            if not img.getbbox():
+                logger.info("Screen Capture Didn't get Capturing anything, sleeping.")
+                time.sleep(1)
+                continue
 
             import random  # Ensure this is imported at the top of the file if not already
             rand_int = random.randint(1, 20)  # Executes only once out of 10 times
@@ -735,7 +753,12 @@ class ScreenshotThread(threading.Thread):
 
             cropped_sections = []
             for area in self.areas:
-                cropped_sections.append(img.crop((area[0], area[1], area[0] + area[2], area[1] + area[3])))
+                # Ensure crop coordinates are within image bounds
+                left = max(0, area[0])
+                top = max(0, area[1])
+                right = min(img.width, area[0] + area[2])
+                bottom = min(img.height, area[1] + area[3])
+                cropped_sections.append(img.crop((left, top, right, bottom)))
 
             if len(cropped_sections) > 1:
                 combined_width = max(section.width for section in cropped_sections)
@@ -755,13 +778,44 @@ class ScreenshotThread(threading.Thread):
             if rand_int == 1:
                 img.save(os.path.join(get_temporary_directory(), 'after_crop.png'), 'PNG')
 
-            self.write_result(img)
-            screenshot_event.clear()
+            if last_image and are_images_identical(img, last_image):
+                logger.debug("Captured screenshot is identical to the last one, sleeping.")
+                time.sleep(max(.5, get_ocr_scan_rate()))
+            else:
+                self.write_result(img)
+                screenshot_event.clear()
 
         if self.macos_window_tracker_instance:
             self.macos_window_tracker_instance.join()
         elif self.windows_window_tracker_instance:
             self.windows_window_tracker_instance.join()
+            
+            
+def set_last_image(image):
+    global last_image
+    if image == last_image:
+        return
+    try:
+        if last_image is not None and hasattr(last_image, "close"):
+            last_image.close()
+    except Exception:
+        pass
+    last_image = image
+    
+def are_images_identical(img1, img2):
+    if None in (img1, img2):
+        return img1 == img2
+
+    try: 
+        img1 = np.array(img1)
+        img2 = np.array(img2)
+    except Exception:
+        logger.warning("Failed to convert images to numpy arrays for comparison.")
+        # If conversion to numpy array fails, consider them not identical
+        return False
+    
+    return (img1.shape == img2.shape) and np.array_equal(img1, img2)
+
 
 # Use OBS for Screenshot Source (i.e. Linux)
 class OBSScreenshotThread(threading.Thread):
@@ -771,6 +825,9 @@ class OBSScreenshotThread(threading.Thread):
         self.interval = interval
         self.obs_client = None
         self.websocket = None
+        self.current_source = None
+        self.current_source_name = None
+        self.current_scene = None
         self.width = width
         self.height = height
         self.use_periodic_queue = not screen_capture_on_combo
@@ -782,46 +839,74 @@ class OBSScreenshotThread(threading.Thread):
             image_queue.put((result, True))
 
     def connect_obs(self):
-        try:
-            import obsws_python as obs
-            self.obs_client = obs.ReqClient(
-                host=get_config().obs.host,
-                port=get_config().obs.port,
-                password=get_config().obs.password,
-                timeout=10
-            )
-            logger.info("Connected to OBS WebSocket.")
-        except Exception as e:
-            logger.error(f"Failed to connect to OBS: {e}")
-            self.obs_client = None
+        import GameSentenceMiner.obs as obs
+        obs.connect_to_obs_sync()
+
 
     def run(self):
+        global last_image
         import base64
         import io
         from PIL import Image
         import GameSentenceMiner.obs as obs
 
-        loop = asyncio.new_event_loop()
-        asyncio.set_event_loop(loop)
+        def init_config(source=None, scene=None):
+            obs.update_current_game()
+            self.current_source = source if source else obs.get_active_source()
+            self.current_source_name = self.current_source.get("sourceName") or None
+            self.current_scene = scene if scene else obs.get_current_game()
+            self.ocr_config = get_scene_ocr_config()
+            if not self.ocr_config:
+                logger.error("No OCR config found for the current scene.")
+                return
+            self.ocr_config.scale_to_custom_size(self.width, self.height)
+
+        # Register a scene switch callback in obsws
+        def on_scene_switch(scene):
+            logger.info(f"Scene switched to: {scene}. Loading new OCR config.")
+            init_config(scene=scene)
+
+        asyncio.run(obs.register_scene_change_callback(on_scene_switch))
 
         self.connect_obs()
-        self.ocr_config.scale_to_custom_size(self.width, self.height)
-        current_source = obs.get_active_source()
-        current_source_name = current_source.get('sourceName') if isinstance(current_source, dict) else None
-
+        init_config()
+        start = time.time()
         while not terminated:
-            try:
-                response = self.obs_client.get_source_screenshot(
-                    name=current_source_name,
-                    img_format='png',
-                    quality=75,
-                    width=self.width,
-                    height=self.height,
-                )
+            if time.time() - start > 5:
+                if not self.obs_client:
+                    self.connect_obs()
+                else:
+                    try:
+                        self.obs_client.get_version()
+                    except Exception as e:
+                        logger.error(f"Lost connection to OBS: {e}")
+                        self.obs_client = None
+                        self.connect_obs()
+            if not screenshot_event.wait(timeout=0.1):
+                continue
 
-                if response.image_data:
-                    image_data = base64.b64decode(response.image_data.split(",")[1])
-                    img = Image.open(io.BytesIO(image_data)).convert("RGBA")
+            if not self.ocr_config:
+                logger.info("No OCR config found for the current scene. Waiting for scene switch.")
+                time.sleep(1)
+                continue
+            
+            if not self.current_source_name:
+                obs.update_current_game()
+                self.current_source = obs.get_active_source()
+                self.current_source_name = self.current_source.get("sourceName") or None
+
+            try:
+                if not self.current_source_name:
+                    logger.error("No active source found in the current scene.")
+                    time.sleep(1)
+                    continue
+                img = obs.get_screenshot_PIL(source_name=self.current_source_name, width=self.width, height=self.height, img_format='jpg', compression=90)
+
+                if img is not None:
+                    if not img.getbbox():
+                        logger.info("OBS Not Capturing anything, sleeping.")
+                        time.sleep(1)
+                        continue
 
                     for rectangle in self.ocr_config.rectangles:
                         if rectangle.is_excluded:
@@ -832,7 +917,12 @@ class OBSScreenshotThread(threading.Thread):
                     cropped_sections = []
                     for rectangle in [r for r in self.ocr_config.rectangles if not r.is_excluded]:
                         area = rectangle.coordinates
-                        cropped_sections.append(img.crop((area[0], area[1], area[0] + area[2], area[1] + area[3])))
+                        # Ensure crop coordinates are within image bounds
+                        left = max(0, area[0])
+                        top = max(0, area[1])
+                        right = min(img.width, area[0] + area[2])
+                        bottom = min(img.height, area[1] + area[3])
+                        cropped_sections.append(img.crop((left, top, right, bottom)))
 
                     if len(cropped_sections) > 1:
                         combined_width = max(section.width for section in cropped_sections)
@@ -846,17 +936,20 @@ class OBSScreenshotThread(threading.Thread):
                         img = combined_img
                     elif cropped_sections:
                         img = cropped_sections[0]
-
-                    self.write_result(img)
+                        
+                    if last_image and are_images_identical(img, last_image):
+                        logger.debug("Captured screenshot is identical to the last one, sleeping.")
+                        time.sleep(max(.5, get_ocr_scan_rate()))
+                    else:
+                        self.write_result(img)
+                        screenshot_event.clear()
                 else:
                     logger.error("Failed to get screenshot data from OBS.")
 
             except Exception as e:
-                logger.error(f"An unexpected error occurred with OBS connection: {e}")
+                logger.error(f"An unexpected error occurred during OBS Capture : {e}", exc_info=True)
                 continue
 
-            time.sleep(self.interval)
-
 class AutopauseTimer:
     def __init__(self, timeout):
         self.stop_event = threading.Event()
@@ -919,6 +1012,22 @@ def engine_change_handler(user_input='s', is_combo=True):
         logger.opt(ansi=True).info(f'Switched to <{engine_color}>{new_engine_name}</{engine_color}>!')
 
 
+def engine_change_handler_name(engine):
+    global engine_index
+    old_engine_index = engine_index
+
+    for i, instance in enumerate(engine_instances):
+        if instance.name.lower() in engine.lower():
+            engine_index = i
+            break
+
+    if engine_index != old_engine_index:
+        new_engine_name = engine_instances[engine_index].readable_name
+        notifier.send(title='owocr', message=f'Switched to {new_engine_name}')
+        engine_color = config.get_general('engine_color')
+        logger.opt(ansi=True).info(f'Switched to <{engine_color}>{new_engine_name}</{engine_color}>!')
+
+
 def user_input_thread_run():
     def _terminate_handler():
         global terminated
@@ -1034,8 +1143,8 @@ def process_and_write_results(img_or_path, write_to=None, last_result=None, filt
     if res:
         if filtering:
             text, orig_text = filtering(text, last_result)
-        if lang == "ja" or lang == "zh":
-            text = post_process(text, keep_blank_lines=keep_new_lines)
+        if get_ocr_language() == "ja" or get_ocr_language() == "zh":
+            text = post_process(text, keep_blank_lines=get_ocr_keep_newline())
         logger.opt(ansi=True).info(f'Text recognized in {end_time - start_time:0.03f}s using <{engine_color}>{engine_instance.readable_name}</{engine_color}>: {text}')
         if notify and config.get_general('notifications'):
             notifier.send(title='owocr', message='Text recognized: ' + text)
@@ -1086,18 +1195,16 @@ def run(read_from=None,
         screen_capture_exclusions=None,
         screen_capture_window=None,
         screen_capture_delay_secs=None,
-        screen_capture_only_active_windows=None,
         screen_capture_combo=None,
         stop_running_flag=None,
         screen_capture_event_bus=None,
         text_callback=None,
-        language=None,
         monitor_index=None,
         ocr1=None,
         ocr2=None,
         gsm_ocr_config=None,
         furigana_filter_sensitivity=None,
-        keep_line_breaks=False,
+        config_check_thread=None
         ):
     """
     Japanese OCR client
@@ -1132,8 +1239,8 @@ def run(read_from=None,
     if screen_capture_area is None:
         screen_capture_area = config.get_general('screen_capture_area')
 
-    if screen_capture_only_active_windows is None:
-        screen_capture_only_active_windows = config.get_general('screen_capture_only_active_windows')
+    # if screen_capture_only_active_windows is None:
+    #     screen_capture_only_active_windows = config.get_general('screen_capture_only_active_windows')
 
     if screen_capture_exclusions is None:
         screen_capture_exclusions = config.get_general('screen_capture_exclusions')
@@ -1159,9 +1266,6 @@ def run(read_from=None,
     if write_to is None:
         write_to = config.get_general('write_to')
 
-    if language is None:
-        language = config.get_general('language', "ja")
-
     logger.configure(handlers=[{'sink': sys.stderr, 'format': config.get_general('logger_format')}])
 
     if config.has_config:
@@ -1173,14 +1277,10 @@ def run(read_from=None,
 
     global engine_instances
     global engine_keys
-    global lang
-    global keep_new_lines
-    lang = language
     engine_instances = []
     config_engines = []
     engine_keys = []
     default_engine = ''
-    keep_new_lines = keep_line_breaks
 
     if len(config.get_general('engines')) > 0:
         for config_engine in config.get_general('engines').split(','):
@@ -1194,7 +1294,7 @@ def run(read_from=None,
             if config.get_engine(engine_class.name) == None:
                 engine_instance = engine_class()
             else:
-                engine_instance = engine_class(config.get_engine(engine_class.name), lang=lang)
+                engine_instance = engine_class(config.get_engine(engine_class.name), lang=get_ocr_language())
 
             if engine_instance.available:
                 engine_instances.append(engine_instance)
@@ -1270,7 +1370,7 @@ def run(read_from=None,
         global txt_callback
         txt_callback = text_callback
 
-    if 'screencapture' in (read_from, read_from_secondary) or 'obs' in (read_from, read_from_secondary):
+    if any(x in ('screencapture', 'obs') for x in (read_from, read_from_secondary)):
         global screenshot_event
         global take_screenshot
         if screen_capture_combo != '':
@@ -1285,7 +1385,7 @@ def run(read_from=None,
         last_result = ([], engine_index)
 
         screenshot_event = threading.Event()
-        screenshot_thread = ScreenshotThread(screen_capture_area, screen_capture_window, screen_capture_exclusions, screen_capture_only_active_windows, screen_capture_areas, screen_capture_on_combo)
+        screenshot_thread = ScreenshotThread(screen_capture_area, screen_capture_window, screen_capture_exclusions, screen_capture_areas, screen_capture_on_combo)
         screenshot_thread.start()
         filtering = TextFiltering()
         read_from_readable.append('screen capture')
@@ -1325,8 +1425,12 @@ def run(read_from=None,
         read_from_readable.append(f'directory {read_from_path}')
 
     if len(key_combos) > 0:
-        key_combo_listener = keyboard.GlobalHotKeys(key_combos)
-        key_combo_listener.start()
+        try:
+            from pynput import keyboard
+            key_combo_listener = keyboard.GlobalHotKeys(key_combos)
+            key_combo_listener.start()
+        except ImportError:
+            pass
 
     if write_to in ('clipboard', 'websocket', 'callback'):
         write_to_readable = write_to
@@ -1347,6 +1451,13 @@ def run(read_from=None,
     if screen_capture_combo:
         logger.opt(ansi=True).info(f'Manual OCR Running... Press <{engine_color}>{screen_capture_combo.replace("<", "").replace(">", "")}</{engine_color}> to run OCR')
 
+    def handle_config_changes(changes):
+        nonlocal last_result
+        if any(c in changes for c in ('ocr1', 'ocr2', 'language', 'furigana_filter_sensitivity')):
+            last_result = ([], engine_index)
+            engine_change_handler_name(get_ocr_ocr1())
+    config_check_thread.add_callback(handle_config_changes)
+
     while not terminated:
         ocr_start_time = datetime.now()
         start_time = time.time()
@@ -1361,7 +1472,7 @@ def run(read_from=None,
                 pass
 
         if (not img) and process_screenshots:
-            if (not paused) and (not screenshot_thread or (screenshot_thread.screencapture_window_active and screenshot_thread.screencapture_window_visible)) and (time.time() - last_screenshot_time) > screen_capture_delay_secs:
+            if (not paused) and (not screenshot_thread or (screenshot_thread.screencapture_window_active and screenshot_thread.screencapture_window_visible)) and (time.time() - last_screenshot_time) > get_ocr_scan_rate():
                 screenshot_event.set()
                 img = periodic_screenshot_queue.get()
                 filter_img = True
@@ -1374,7 +1485,7 @@ def run(read_from=None,
             break
         elif img:
             if filter_img:
-                res, _ = process_and_write_results(img, write_to, last_result, filtering, notify, ocr_start_time=ocr_start_time, furigana_filter_sensitivity=furigana_filter_sensitivity)
+                res, _ = process_and_write_results(img, write_to, last_result, filtering, notify, ocr_start_time=ocr_start_time, furigana_filter_sensitivity=get_ocr_furigana_filter_sensitivity())
                 if res:
                     last_result = (res, engine_index)
             else:
@@ -1398,8 +1509,10 @@ def run(read_from=None,
         directory_watcher_thread.join()
     if unix_socket_server:
         unix_socket_server.shutdown()
-        unix_socket_server_thread.join()
+        unix_socket_server.join()
     if screenshot_thread:
         screenshot_thread.join()
     if key_combo_listener:
         key_combo_listener.stop()
+    if config_check_thread:
+        config_check_thread.join()
